!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! init_state.f90	Initialize the model state by allocating and evaluating those global
!			arrays defined in the dyn_vars module.
!
!			Called by PROGRAM qgam.
! 
! twnh 14Aug01 & pdw 5Jun02
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE init_state

! load global variables
USE grid_params
USE phys_params
USE dyn_vars

IMPLICIT NONE

! declaration of local variables
INTEGER		:: i, j, k, l							! indices

WRITE(*,*) ' init_state: Initializing state arrays.'

! allocate storage space for permanent arrays
ALLOCATE (	q(		N_rad, N_azim, N_layer, 3),	&	! QGPV field at 3 time levels
		psi(		N_rad, N_azim, N_layer, 3),	&	! streamfunction field at 3 time levels
		q_tend(		N_rad, N_azim, N_layer),	&	! tendency in q field
		q_relax(	N_rad, N_azim, N_layer),	&	! field to relax q to
		psi_relax(	N_rad, N_azim, N_layer),	&	! field to relax psi to
		random(		N_rad, N_azim),			&	! random number array for stochastic forcing
		top_forc_coeff(	N_rad, N_azim, N_layer)		)	! coefficient for topographic forcing

! get initial state at two time slices, either by reading pickup files from disk or by calculation
IF (start_step /= 1) THEN					! load q and psi pickup files from disk

	CALL read_pu_fields(start_step)
	start_step = start_step + 1				! don't repeat the starting step (done when pickup was written)

ELSE 								! calculate q and psi internally

	WRITE(*,*) ' init_state: Generating initial PV field made of random numbers'
	CALL RANDOM_SEED()		! a random number based on the execution date and time is assigned to the seed
					! the execution time is stored in seconds, so if the code is executed more than once
					! within the same second, the same random initial fields will be obtained
	CALL RANDOM_NUMBER(q(:,:,:,:))	! the harvest is real pseudorandom numbers with a uniform distribution in the interval (0,1)
	q(:,:,:,:) = initial_amplitude*(2.0_wp*q(:,:,:,:)-1.0_wp)	! shift to the interval (-initial_amplitude,initial_amplitude)

	q(1,:,:,:) = 0.0_wp			! } to avoid noisy PV field at boundaries
	q(N_rad,:,:,:) = 0.0_wp			! } near start of run

	DO k = 1, N_layer			! set mean PV to zero in each layer and at all three steps
		DO l = 1, 3
			q(:,:,k,l) = q(:,:,k,l) - SUM(area_element*q(:,:,k,l))/area
		END DO
	END DO

	WRITE(*,*) ' init_state: Calculating psi field from PV field'
	CALL solver(q(:,:,:,prev_step), psi(:,:,:,prev_step))
	CALL solver(q(:,:,:,this_step), psi(:,:,:,this_step))
	CALL solver(q(:,:,:,next_step), psi(:,:,:,next_step))

END IF

! read relaxation fields q_relax and psi_relax from disk
CALL read_forcing_fields

! evaluate topographic forcing fields
DO i=1, N_rad
        DO j=1, N_azim
                top_forc_coeff(i, j, 1) = - ( f * s_top ) / ( H * rad_dist(i) )
                top_forc_coeff(i, j, 2) = + ( f * s_bot ) / ( H * rad_dist(i) )
        END DO
END DO

END SUBROUTINE init_state
