!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! solver.f90	Subroutine to solve Helmholtz equation in cylindrical polar co-ordinates.          
!		Subroutine calculates:
!					psi_out = solver (q_in).
!
!		Called by PROGRAM qgam and by SUBROUTINE init_state.
! 
! pdw & twnh 13Aug01
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE solver(q_in, psi_out)

! NAG routines used
USE nag_fft, ONLY : nag_fft_1d_real
USE nag_gen_bnd_lin_sys, ONLY : nag_gen_bnd_lin_sol_fac

! load global variables
USE grid_params
USE phys_params
USE solver_vars

IMPLICIT NONE

! declaration of local variables
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(IN)	:: q_in
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(OUT)	:: psi_out
REAL (KIND=wp), DIMENSION(N_rad, N_azim)			:: q_bt, q_bc, psi_bt, psi_bc
COMPLEX (KIND=wp), DIMENSION(N_rad, N_azim)			:: q_bt_FT, q_bc_FT, psi_bt_FT, psi_bc_FT
COMPLEX (KIND=wp), DIMENSION(N_rad)				:: RHS_bt, RHS_bc
INTEGER								:: i, j 

WRITE(*,*) ' solver...'

! split q_in field into barotropic and baroclinic parts
q_bt = q_in(:,:,1) + q_in(:,:,2)
q_bc = itcc*(q_in(:,:,2) - q_in(:,:,1))

! create matrix, each row of which contains the Fourier components of the corresponding row of q_bt, and q_bc
q_bt_FT = nag_fft_1d_real(q_bt, inverse=.FALSE., trig=fft_coeffs)/sqrt(REAL(N_azim, KIND=wp))
q_bc_FT = nag_fft_1d_real(q_bc, inverse=.FALSE., trig=fft_coeffs)/sqrt(REAL(N_azim, KIND=wp))

! calculate psi_bt_FT, psi_bc_FT from q_bt_FT, q_bc_FT (only for columns 1 to N_azim/2+1 inclusive)
DO j=1, N_azim/2 + 1   
	RHS_bt(:) = q_bt_FT(:,j)*(delta_r**2)
	RHS_bt(1) = 0.0_wp
	RHS_bt(N_rad) = 0.0_wp
	RHS_bc(:) = q_bc_FT(:,j)*(delta_r**2)
	RHS_bc(1) = 0.0_wp
	RHS_bc(N_rad) = 0.0_wp
	CALL nag_gen_bnd_lin_sol_fac(1, a_bt_fac(j,:,:), pivot_bt(j,:), RHS_bt)
	CALL nag_gen_bnd_lin_sol_fac(1, a_bc_fac(j,:,:), pivot_bc(j,:), RHS_bc)
	psi_bt_FT(:, j) = RHS_bt(:)
	psi_bc_FT(:, j) = RHS_bc(:)
END DO

! modify barotropic mean flow correction at one boundary
psi_bt_FT(N_rad, 1) = psi_bt_FT(N_rad - 1, 1)

! fill in remaining columns of psi_bt_FT, psi_bc_FT such that each row is an
! Hermitian series, ensuring that each row of psi_bt, psi_bc is a real series
DO i=1, N_azim/2-1
	psi_bt_FT(:, N_azim/2 + 1 + i) = CONJG(psi_bt_FT(:, N_azim/2 + 1 - i))
	psi_bc_FT(:, N_azim/2 + 1 + i) = CONJG(psi_bc_FT(:, N_azim/2 + 1 - i))
END DO

psi_bt = nag_fft_1d_real(psi_bt_FT, inverse=.TRUE., trig=fft_coeffs)*sqrt(REAL(N_azim, KIND=wp))
psi_bc = nag_fft_1d_real(psi_bc_FT, inverse=.TRUE., trig=fft_coeffs)*sqrt(REAL(N_azim, KIND=wp))

psi_out(:,:,1) = (psi_bt - psi_bc)/2.0_wp
psi_out(:,:,2) = (psi_bt + psi_bc)/2.0_wp

END SUBROUTINE solver
